import {
  environmentToExpectedPublicKey
} from "./chunk-O7PUADIE.js";
import {
  usageOriginToPrefix
} from "./chunk-NGSNVE6K.js";
import {
  CROSSMINT_API_KEY_SIGNER_PUBLICKEY_PROD,
  CROSSMINT_API_KEY_SIGNER_PUBLICKEY_STAGING,
  CROSSMINT_DEV_URL,
  CROSSMINT_PROD_URL,
  CROSSMINT_STG_URL
} from "./chunk-7UFNMI6J.js";
import {
  APIKeyEnvironmentPrefix,
  APIKeyUsageOrigin,
  APIKeyUsageOriginPrefix
} from "./chunk-CSYWDKB6.js";
import {
  __spreadProps,
  __spreadValues
} from "./chunk-JPRRZPBL.js";

// src/apiKey/validateAPIKey.ts
import base58 from "bs58";
import nacl from "tweetnacl";

// src/apiKey/validateAPIKeyPrefix.ts
function validateAPIKeyPrefix(apiKey, expectations) {
  if (isOldAPIKey(apiKey)) {
    return {
      isValid: false,
      message: "Old API key format detected. Please create a new API key from the Crossmint console."
    };
  }
  const usageOrigin = getUsageOriginForKey(apiKey);
  if (usageOrigin == null) {
    return {
      isValid: false,
      message: `Malformed API key. Must start with '${APIKeyUsageOriginPrefix.CLIENT}' or '${APIKeyUsageOriginPrefix.SERVER}'.`
    };
  }
  if ((expectations == null ? void 0 : expectations.usageOrigin) != null && usageOrigin !== expectations.usageOrigin) {
    return {
      isValid: false,
      message: `Disallowed API key. You passed a ${usageOrigin} API key, but a ${expectations.usageOrigin} API key is required.`
    };
  }
  const environment = getEnvironmentForKey(apiKey);
  if (environment == null) {
    return {
      isValid: false,
      message: `Malformed API key. Must have a valid environment: '${APIKeyEnvironmentPrefix.DEVELOPMENT}', '${APIKeyEnvironmentPrefix.STAGING}' or '${APIKeyEnvironmentPrefix.PRODUCTION}'.`
    };
  }
  if ((expectations == null ? void 0 : expectations.environment) != null && environment !== expectations.environment) {
    return {
      isValid: false,
      message: `Disallowed API key. You passed a ${environment} API key, but a ${expectations.environment} API key is required.`
    };
  }
  return {
    isValid: true,
    usageOrigin,
    environment,
    prefix: `${usageOriginToPrefix(usageOrigin)}_${environment}`
  };
}
function isOldAPIKey(apiKey) {
  return apiKey.startsWith("sk_live") || apiKey.startsWith("sk_test");
}
function getUsageOriginForKey(apiKey) {
  if (apiKey.startsWith(APIKeyUsageOriginPrefix.CLIENT + "_")) {
    return APIKeyUsageOrigin.CLIENT;
  } else if (apiKey.startsWith(APIKeyUsageOriginPrefix.SERVER + "_")) {
    return APIKeyUsageOrigin.SERVER;
  }
  return null;
}
function getEnvironmentForKey(apiKey) {
  const keyWithoutUsageOrigin = apiKey.slice(3);
  if (keyWithoutUsageOrigin.startsWith(APIKeyEnvironmentPrefix.DEVELOPMENT + "_")) {
    return APIKeyEnvironmentPrefix.DEVELOPMENT;
  } else if (keyWithoutUsageOrigin.startsWith(APIKeyEnvironmentPrefix.STAGING + "_")) {
    return APIKeyEnvironmentPrefix.STAGING;
  } else if (keyWithoutUsageOrigin.startsWith(APIKeyEnvironmentPrefix.PRODUCTION + "_")) {
    return APIKeyEnvironmentPrefix.PRODUCTION;
  }
  return null;
}

// src/apiKey/validateAPIKey.ts
function validateAPIKey(apiKey, expectations) {
  const prefixValidationResult = validateAPIKeyPrefix(apiKey, expectations);
  if (!prefixValidationResult.isValid) {
    return prefixValidationResult;
  }
  const { prefix } = prefixValidationResult;
  const { keyData, keyDataWithoutPrefix, signature } = decodeAPIKeyAndGetParts(apiKey, prefix);
  const isSignatureValid = validateAPIKeySignature(keyData, signature, expectations);
  if (!isSignatureValid) {
    return {
      isValid: false,
      message: "Invalid API key. Failed to validate signature"
    };
  }
  const [projectId] = keyDataWithoutPrefix.split(".");
  return __spreadProps(__spreadValues({}, prefixValidationResult), {
    projectId
  });
}
function decodeAPIKeyAndGetParts(apiKey, prefix) {
  const base58EncodedPart = apiKey.slice(`${prefix}_`.length);
  const decodedPartUint8 = base58.decode(base58EncodedPart);
  const decodedPartString = new TextDecoder().decode(decodedPartUint8);
  const [keyDataWithoutPrefix, signature] = decodedPartString.split(":");
  const keyData = `${prefix}.${keyDataWithoutPrefix}`;
  return {
    keyData,
    keyDataWithoutPrefix,
    signature
  };
}
function validateAPIKeySignature(keyData, signature, expectations) {
  const expectedPublicKey = environmentToExpectedPublicKey(expectations == null ? void 0 : expectations.environment);
  function _validateSignature(expectedPublicKey2) {
    try {
      const signatureBytes = base58.decode(signature);
      const keyDataBytes = new TextEncoder().encode(keyData);
      const publicKeyBytes = base58.decode(expectedPublicKey2);
      return nacl.sign.detached.verify(keyDataBytes, signatureBytes, publicKeyBytes);
    } catch (e) {
      console.error("Failed to validate API key signature");
      return false;
    }
  }
  if (expectedPublicKey != null) {
    return _validateSignature(expectedPublicKey);
  }
  return _validateSignature(CROSSMINT_API_KEY_SIGNER_PUBLICKEY_PROD) || _validateSignature(CROSSMINT_API_KEY_SIGNER_PUBLICKEY_STAGING);
}

// src/apiKey/utils/validateApiKeyAndGetCrossmintBaseUrl.ts
var urlMap = {
  development: CROSSMINT_DEV_URL,
  staging: CROSSMINT_STG_URL,
  production: CROSSMINT_PROD_URL
};
function validateApiKeyAndGetCrossmintBaseUrl(apiKey) {
  const result = validateAPIKey(apiKey);
  if (!result.isValid) {
    throw new Error("Crossmint API key is invalid");
  }
  return urlMap[result.environment];
}

export {
  validateApiKeyAndGetCrossmintBaseUrl,
  validateAPIKeyPrefix,
  getEnvironmentForKey,
  validateAPIKey
};
//# sourceMappingURL=chunk-S4L4GIE4.js.map